;=======================================================================================
;	ENDURANCE BOARD PROGRAM V1.0
;	AUTHOR: MOHD NAUFAL BIN AHMAD KHAIRI
;	DESCRIPTION: the program is used to do endurance testing 
		     and applying several APDU's based on
;		     ISO 7816 contact smart card
;========================================================================================

IO  equ p2.0
RST	equ p2.1
OK 	equ p3.6
xOK	equ p3.7
sw1 equ p2.4
sw2 equ	p2.5
sw3 equ p2.6
sw4 equ p2.7
count_bit	equ	p2.3
;LED equ p3.2
LCD_data equ P1    ;LCD Data port
LCD_D7   equ P1.7  ;LCD D7/Busy Flag
LCD_rs   equ P3.3  ;LCD Register Select
LCD_rw   equ P3.4  ;LCD Read/Write
LCD_en   equ P3.5  ;LCD Enable
keyport equ P0                  ;Keypad port connected here
col1 equ P0.0                   ;Column 1
col2 equ P0.1                   ;Column 2
col3 equ P0.2                   ;Column 3
col4 equ P0.3                   ;Column 4
bit1 bit 20h
endurance_bit bit 28h
switch_mode bit 27h

org 0000h
jmp	main

;org 0003h
;jmp	pause_isr

org 000Bh
jmp timer_isr

timer_isr:
	push	acc
	push	psw
	push	00h
	push	01h
	push	02h
	push	03h
	push	04h
	push	05h
	push	06h
	push	07h
	djnz	55h,return
	pop		acc
	pop		psw
	pop		00h
	pop		01h
	pop		02h
	pop		03h
	pop		04h
	pop		05h
	pop		06h
	pop		07h
	clr 	EA
	clr		ET0
	clr		tr0
	jmp		sendPTS
return:
	pop		acc
	pop		psw
	pop		00h
	pop		01h
	pop		02h
	pop		03h
	pop		04h
	pop		05h
	pop		06h
	pop		07h
	reti

main:

;========================initializing===================================
	call LCD_init		  	;initialize LCD, very important
	mov   a,#01H         	;load #01h to acc before call LCD_command
	lcall LCD_command		;send the command to clear LCD
	mov   dptr,#my_string3  ;load 'INIT LCD' string to dptr
	lcall LCD_sendstring    ;display 'INIT LCD'
	call delay			    ;delay for
	call delay				;display the string for
	call delay				;a while
	mov   a,#01H         	
	lcall LCD_command
	mov   dptr,#my_string   
	lcall LCD_sendstring    
	call delay
	call delay
	call delay
	clr	  OK				 ;light on the GREEN LED
	call delay
	call delay
	call delay
	setb  OK				 ;light off the GREEN LED
	clr   xOK				 ;light on the RED LED
	call delay
	call delay
	call delay
	setb  xOK				 ;light off the RED LED
	setb	count_bit		   	;set count bit =1
;===========================auto-reset===================================================
;after initiate all the system, the program is jump to this point						= 
;where the system will wait for smartcard to be inserted.								= 
;There is a switch connected to pin2.2 that will detect the smartcard is inserted or not= 
;========================================================================================																						 ;
auto:
	mov   a,#01H         		;01H is command for clearing LCD
	lcall LCD_command    		;Send the command
	mov   dptr,#my_string13   	;my_string is the label where the string is stored
	lcall LCD_sendstring    	;'INSERT CARD' displayed
auto2:	
	jnb	p2.2,auto2				;check if card not inserted
								;looping until card inserted
;===============================ATR=====================================================
;at this routine area, timer0 interrupt will be activated to detect the ATR bytes for a while
;and just afer the ATR bytes is confirmed true, the timer0 ISR will jump to sendPTS routine
;at this point also, the ATR bytes will be dislayed through LCD
;
;=======================================================================================
ATR:
	setb	EA					;enable global interrupt
	setb	30h
	clr		31h
	clr		27h		 			;for keypad mode 2 purpose
	clr		28h		 			;for counting purpose only
	mov		55h,#25				;this value will determine the ATR timer interrupt period
;=========================initialize timer 0============================
	mov	tmod,#01h	;16-bit mode
;=======================================================================
	mov   a,#01H         		;01H is command for clearing LCD
	lcall LCD_command    		;Send the command
	mov   dptr,#my_string4   
	lcall LCD_sendstring    	;'ATR:'
	mov   	a,#0C0H         	;jump to 2nd line
	lcall 	LCD_command     	;Send the command to LCD
	mov		r6,#9
	clr		RST					;start reset to pin RST of smart card
	call	etu_9600			;level 0 for a while
	setb	tr0					;start timer 0
	setb	ET0					;enable timer 0 interrupt
	setb	RST				    ;set RST pin of smart card
	call	checkATR		  	;sense ATR bytes
sendPTS:
	jb		31h,remove_card		;31h is the bit address to verify if the ATR is true or not
	acall   delayPTS			;delay for a while before send PTS bytes
	acall	PTS 				;send PTS
	acall	delayPTS			;delay for a while to give smartcard response the PTS	
again:
	setb	xOK
	mov   a,#80H         
	call LCD_command   
again2:
	mov   dptr,#my_string11   ;my_string is the label where the string is stored
	call LCD_sendstring    ;INSERT APDU
;============================pause interrupt===============================
	;	setb	EX0
	;	setb	EA		;enable back global interupt
	;	clr		IT0		;enable low-level triggered interrupt on INT0 (P3.2)
;============================keypad========================================
;at this point, program start to activate keypad function
;
;
;==========================================================================
keypad:
		mov keyport,#0F0H        ;Make rows as o/p and col as i/p
get_key:
;------------row1-------------------
row1:
        mov keyport,#0EFH       ;make Row1 low
		jb	col1,nextcol
		jnb	col1,$
		setb	xOK
		setb	OK
		mov   a,#01H         	;01H is command for clearing LCD
		call LCD_command    	;Send the command
		jbc		switch_mode,endurance
		mov   dptr,#my_string34   ;erase mode      
		call LCD_sendstring   	;Send data
		acall	keygen38k
		jmp	nextcol

remove_card:
		jb	p2.2,$
		jmp	auto

endurance:
		mov   dptr,#my_string21   ;endurance mode      
		lcall LCD_sendstring   ;Send data
		mov   a,#0C0H         	;jump to 2nd line
		lcall LCD_command   	;Send the command
		lcall	countX
		lcall	countY
		lcall	count_start	
		; new added line:jnb
		jnb		p2.2, stop   ;check whether there is still card or not	
nextcol:
		jb	col2,nextcol1
		jnb	col2,$
		setb	xOK
		setb	OK
		mov   a,#01H         	;01H is command for clearing LCD
		lcall LCD_command    	;Send the command
		jbc		switch_mode,endurance_bist
		mov   	dptr,#my_string16         
		lcall LCD_sendstring  	 ;Send data
		acall	CPU38k
		jmp	nextcol1
			
endurance_bist:
		mov   	dptr,#my_string33         
		lcall LCD_sendstring  	 ;Send data
		acall	XX_data
		acall	check_parityXX
		acall	YY_data
		acall	check_parityYY
		mov   a,#0C0H         	;01H is command for clearing LCD
		lcall LCD_command    	;Send the command
		mov   	dptr,#my_string19   
		call 	LCD_sendstring  
		call	endurance_bist_38k
						
nextcol1:
		jb	col3,nextcol11
		jnb	col3,$
		setb	xOK
		setb	OK
		mov   a,#01H         	;01H is command for clearing LCD
		lcall LCD_command    	;Send the command
		jbc		switch_mode,SHA
		mov   	dptr,#my_string43         
		lcall LCD_sendstring   	;Send data
		call	Timer38k
		jmp		nextcol11

SHA:
		mov   	dptr,#my_string49   
		call 	LCD_sendstring  
		call	SHA_38k		
		
nextcol11:
		jb	col4,row2
		jnb	col4,$
		setb	xOK
		setb	OK
		mov   a,#01H         	;01H is command for clearing LCD
		lcall LCD_command    	;Send the command
		jbc		switch_mode,Erase_byte
		mov   	dptr,#my_string44         
		lcall LCD_sendstring   	;Send data	
		call Watchdog38k
		jmp		row2

Erase_byte:
		mov   	dptr,#my_string50   
		call 	LCD_sendstring
		acall	XX_data
		acall	check_parityXX
		acall	YY_data
		acall	check_parityYY 
		acall	ZZ_data
		acall	check_parityZZ 
		call	Erase_byte38k
;------------------row2--------------------        
row2:
        mov keyport,#0DFH       ;make Row2 low
		jb	col1,nextcol2		;5
		jnb	col1,$
		setb	xOK
		setb	OK
		mov   a,#01H         	;01H is command for clearing LCD
		lcall LCD_command    	;Send the command
		jbc		switch_mode,loop_1024bit_RSA
		mov   	dptr,#my_string45         
		lcall LCD_sendstring   ;Send data
		acall	SramBist38k
		jmp		nextcol2

loop_1024bit_RSA:
		mov   	dptr,#my_string52         
		lcall 	LCD_sendstring   ;Send data
		call	loop_1024bit_RSA38k

nextcol2:
		jb	col2,nextcol22		;6
		jnb	col2,$
		setb	xOK
		setb	OK
		mov   a,#01H         	;01H is command for clearing LCD
		lcall LCD_command    	;Send the command
		jbc		switch_mode,long_delay
		mov   	dptr,#my_string14	;erase all 
		lcall LCD_sendstring   ;Send data
		acall	erase38k
		jmp		nextcol22

long_delay:
		mov   	dptr,#my_string53         
		lcall 	LCD_sendstring   ;Send data
		call	long_delay38k

nextcol22:
		jb	col3,nextcol222		;7
		jnb	col3,$
		setb	xOK
		setb	OK
		mov   a,#01H         	;01H is command for clearing LCD
		lcall LCD_command    	;Send the command
		jbc		switch_mode,write_wholeX
		mov   	dptr,#my_string15     ;write byte XX    
		lcall   LCD_sendstring 	    ;Send data
		acall	write38k
		jmp		nextcol222
write_wholeX:
		mov   	dptr,#my_string54   
		call 	LCD_sendstring
		acall	XX_data
		acall	check_parityXX
		call	write_wholeX38k

nextcol222:					   	
		jb	col4,row3			
		jnb	col4,$
		setb	xOK
		setb	OK
		mov   a,#01H         	
		lcall LCD_command    	
		mov   	dptr,#my_string28        
		lcall   LCD_sendstring 	    ;Send data
		acall	XX_data				;wait for user input
		acall	check_parityXX
		acall	writeeven38k
;-------------row3--------------------------------
row3:
        mov keyport,#0BFH       		
		jb	col1,nextcol3				
		jnb	col1,$
		setb	xOK
		setb	OK
		mov   a,#01H         			;01H is command for clearing LCD
		lcall LCD_command    			;Send the command
		mov   	dptr,#my_string29    	     
		lcall   LCD_sendstring 	    	
		acall	XX_data					
		acall	check_parityXX
		acall	writeodd38k
nextcol3:
		jb	col2,nextcol33				
		jnb	col2,$
		setb	xOK
		setb	OK
		mov   a,#01H         			
		lcall LCD_command    			
		mov   	dptr,#my_string27         
		lcall LCD_sendstring
		acall	checksum38k

nextcol33:
		jb	col3,nextcol333				
		jnb	col3,$
		setb	xOK
		setb	OK
		mov   a,#01H         			
		lcall LCD_command    			
		mov   	dptr,#my_string40         
		call LCD_sendstring   			
		call	OTP_CRC_38k
	
nextcol333:
		jb	col4,row4			
		jnb	col4,$
		setb	xOK
		setb	OK
		mov   a,#01H         	
		lcall LCD_command    	
		mov   	dptr,#my_string41 
		lcall LCD_sendstring        
		call	RSA_512b_38k

;----------row4--------------------
row4:
        mov keyport,#07FH       ;make row4 low
		jb	col1,nextcol4		
		jnb	col1,$
		setb	xOK
		setb	OK
		mov   a,#01H         	;01H is command for clearing LCD
		call LCD_command    	;Send the command
		mov   	dptr,#my_string46         
		call LCD_sendstring   	;Send data
		call	DES_CBC38k
nextcol4:
		jb	col2,nextcol44		
		jnb	col2,$
		setb	xOK
		setb	OK
		mov   a,#01H         	;01H is command for clearing LCD
		call LCD_command    	;Send the command
		mov   	dptr,#my_string47         
		lcall LCD_sendstring   ;Send data
		call DES_nonCBC38k
nextcol44:
		jb	col3,nextcol444		
		jnb	col3,$
		setb	xOK
		setb	OK
		mov   a,#01H         	;01H is command for clearing LCD
		call LCD_command    	;Send the command
		mov   	dptr,#my_string35         
		call LCD_sendstring
		acall	MathCo_512bit_38k
nextcol444:
		jb	col4,switchATR			
		jnb	col4,$
		setb	xOK
		setb	OK
		mov   a,#01H         		;01H is command for clearing LCD
		call LCD_command    		;Send the command
		mov   	dptr,#my_string37	;SWITCH CLOCK         
		call LCD_sendstring   		;Send data
		acall	XX_data
		acall	check_parityXX
		acall	YY_data
		acall	check_parityYY
		acall	switch_clock_38k

switchATR:
		jb		sw1,card_out
		jnb		sw1,$
		setb		27h		;set bit address 24
	;	jmp		main

card_out:
		jb 		p2.2,jump
		jnb		p2.2,pulled_out
jump:
		jmp	keypad
pulled_out:
		mov   a,#01H         		;01H is command for clearing LCD
		call LCD_command    		;Send the command
		mov   dptr,#my_string12   	;my_string is the label where the string is stored
		call LCD_sendstring    	;CARD removed
		call delay
		call delay
		call delay
		call delay
		call delay
		call delay			                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          
		call delay
		call delay
		jmp auto

;==========================end of keypad===================================

;==========================APDU Routine====================================
;this routine will have several APDU categorized by their name and types of baud rate
;for ex: erase9600 means APDU for ERASE ALL at 9600 baudrate 
;        & write38k means APDU for WRITE ALL at 38k baudrate    
;
;
;===========================================================================

;===========================9600 mode=================================
/*
erase9600:

	mov r2,#5	;5 character to be sent
	mov	A,#00h	;1mc
	call	put_lo9600	;2mc
	mov	A,#10h
	call	put_hi9600
	mov	A,#00h
	call	put_lo9600
	mov	a,#00h
	call	put_lo9600
	mov	a,#00h
	call	put_lo9600
	ret

write9600:

	mov r2,#5	;5 character to be sent
	mov	a,#00h	;1mc
	call	put_lo9600	;2mc
	mov	a,#12h
	call	put_lo9600
	mov	a,#00h
	call	put_lo9600
	mov	a,#00h
	call	put_lo9600
	mov	a,#00h
	call	put_lo9600
	ret

writeeven9600:

	mov r2,#6	;5 character to be sent
	mov	a,#00h	;1mc
	call	put_lo9600	;2mc
	mov	a,#14h
	call	put_lo9600
	mov	a,#00h
	call	put_lo9600
	mov	a,#00h
	call	put_lo9600
	mov	a,#00h
	call	put_lo9600
	mov	a,#0AAh
	call	put_lo9600
	ret

writeodd9600:
//	mov r5,#255
	mov r2,#6	;5 character to be sent
	mov	a,#00h	;1mc
	call	put_lo9600	;2mc
	mov	a,#16h
	call	put_lo9600
	mov	a,#00h
	call	put_lo9600
	mov	a,#00h
	call	put_lo9600
	mov	a,#00h
	call	put_lo9600
	mov	a,#55h
	call	put_lo9600
	ret
*/
;==============================38k mode================================
SramBist38k:
	mov r2,#5	;5 character to be sent
	mov	A,#00h	;1mc
	call	put_lo38k	;2mc
	mov	A,#0Ah
	call	put_lo38k
	mov	A,#00h
	call	put_lo38k
	mov	a,#00h
	call	put_lo38k
	mov	a,#00h
	call	put_lo38k
	ret

long_delay38k:
	mov r2,#5	;5 character to be sent
	mov	A,#00h	;1mc
	call	put_lo38k	;2mc
	mov	A,#39h
	call	put_lo38k
	mov	A,#00h
	call	put_lo38k
	mov	a,#00h
	call	put_lo38k
	mov	a,#00h
	call	put_lo38k
	ret

loop_1024bit_RSA38k:
	mov r2,#5	;5 character to be sent
	mov	A,#00h	;1mc
	call	put_lo38k	;2mc
	mov	A,#3Bh
	call	put_hi38k
	mov	A,#00h
	call	put_lo38k
	mov	a,#00h
	call	put_lo38k
	mov	a,#00h
	call	put_lo38k
	ret

erase38k:
	mov r2,#5	;5 character to be sent
	mov	A,#00h	;1mc
	call	put_lo38k	;2mc
	mov	A,#10h
	call	put_hi38k
	mov	A,#00h
	call	put_lo38k
	mov	a,#00h
	call	put_lo38k
	mov	a,#00h
	call	put_lo38k
	ret

write38k:
	mov r2,#5	;5 character to be sent
	mov	a,#00h	;1mc
	call	put_lo38k	;2mc
	mov	a,#12h
	call	put_lo38k
	mov	a,#00h
	call	put_lo38k
	mov	a,#00h
	call	put_lo38k
	mov	a,#00h
	call	put_lo38k
	ret
;=========================write even==========================
writeeven38k:
	mov r2,#6	;5 character to be sent
	mov	a,#00h	;1mc
	call	put_lo38k	;2mc
	mov	a,#14h
	call	put_lo38k
	mov	a,#00h
	call	put_lo38k
	mov	a,#00h
	call	put_lo38k
	mov	a,#01h
	call	put_hi38k
data_byte:
	mov		A,63h							 
	jbc		23h,data_byte_hi
	call	put_lo38k
	jmp		nex2
data_byte_hi:
	call	put_hi38k
nex2:					
	;mov	a,#0AAh
	;lcall	put_lo38k
	ret
;=========================write odd==========================
writeodd38k:
	mov r2,#6	;5 character to be sent
	mov	a,#00h	;1mc
	call	put_lo38k	;2mc
	mov	a,#16h
	call	put_hi38k
	mov	a,#00h
	call	put_lo38k
	mov	a,#00h
	call	put_lo38k
	mov	a,#01h
	call	put_hi38k
data_byte2:
	mov		A,63h							 
	jbc		23h,data_byte_hi2
	call	put_lo38k
	jmp		nex3
data_byte_hi2:
	call	put_hi38k
	;mov	a,#55h
	;lcall	put_lo38k
nex3:
	ret
;=========================write whole EEPROM with dataX==========================

write_wholeX38k:
	mov r2,#5	;5 character to be sent
	mov	a,#00h	;1mc
	call	put_lo38k	;2mc
	mov	a,#74h
	call	put_hi38k
data_X:
	mov	a,63h
	jbc		23h,put_hi_dataX
	call	put_lo38k
	jmp		next_dataX
put_hi_dataX:
	call	put_hi38k
next_dataX:
	mov	a,#00h
	call	put_lo38k
	mov	a,#00h
	call	put_lo38k
	ret
;===============================================erase byte==============================
Erase_byte38k:
	mov 	r2,#5				;5 character to be sent
	mov		A,#00h				;1mc
	call	put_lo38k			;2mc
	mov		A,#1Ah
	call	put_hi38k
msb:
	mov		A, 63h
	jbc		23h,put_hi_msb
	call	put_lo38k
	jmp		lsb
put_hi_msb:
	call	put_hi38k			
lsb:
	mov		a, 62h
	jbc		24h,put_hi_lsb
	call	put_lo38k			
	jmp		next_data
put_hi_lsb:
	call	put_hi38k
next_data:	
	mov		a,61h
	jbc		25h,put_hi_data
	call	put_lo38k
	jmp		go_back
put_hi_data:
	call	put_hi38k
go_back:
	ret	
;===============================================DES==============================
DES_CBC38k:
	mov 	r2,#5	;5 character to be sent
	mov		A,#00h	;1mc
	call	put_lo38k	;2mc
	mov		A,#44h
	call	put_lo38k
	mov		A,#00h
	call	put_lo38k
	mov		a,#00h
	call	put_lo38k
	mov		a,#00h
	call	put_lo38k
	ret

DES_nonCBC38k:
	mov 	r2,#5	;5 character to be sent
	mov		A,#00h	;1mc
	call	put_lo38k	;2mc
	mov		A,#46h
	call	put_hi38k
	mov		A,#00h
	call	put_lo38k
	mov		a,#00h
	call	put_lo38k
	mov		a,#00h
	call	put_lo38k
	ret

CPU38k:
	mov 	r2,#5	;5 character to be sent
	mov		A,#00h	;1mc
	call	put_lo38k	;2mc
	mov		A,#02h
	call	put_hi38k
	mov		A,#00h
	call	put_lo38k
	mov		a,#00h
	call	put_lo38k
	mov		a,#00h
	call	put_lo38k
	ret

Timer38k:
	mov 	r2,#5	;5 character to be sent
	mov		A,#00h	;1mc
	call	put_lo38k	;2mc
	mov		A,#04h
	call	put_hi38k
	mov		A,#00h
	call	put_lo38k
	mov		a,#00h
	call	put_lo38k
	mov		a,#00h
	call	put_lo38k
	ret

Watchdog38k:
	mov 	r2,#5	;5 character to be sent
	mov		A,#00h	;1mc
	call	put_lo38k	;2mc
	mov		A,#06h
	call	put_lo38k
	mov		A,#00h
	call	put_lo38k
	mov		a,#00h
	call	put_lo38k
	mov		a,#00h
	call	put_lo38k
	ret

SHA_38k:
	mov 	r2,#5	;5 character to be sent
	mov		A,#00h	;1mc
	call	put_lo38k	;2mc
	mov		A,#4Ch
	call	put_hi38k
	mov		A,#00h
	call	put_lo38k
	mov		a,#00h
	call	put_lo38k
	mov		a,#00h
	call	put_lo38k
	ret
		
checksum38k:
	mov 	r2,#5	;5 character to be sent
	mov		a,#00h	;1mc
	call	put_lo38k	;2mc
	mov		a,#1Eh
	call	put_lo38k
	mov		a,#00h
	call	put_lo38k
	mov		a,#00h
	call	put_lo38k
	mov		a,#02h
	call	put_hi38k
	ret

keygen38k:
	mov 	r2,#5	;5 character to be sent
	mov		A,#00h	;1mc
	call	put_lo38k	;2mc
	mov		A,#01h
	call	put_hi38k
	mov		A,#00h
	call	put_lo38k	;msb
	mov		a,#00h
	call	put_lo38k	;lsb
	mov		a,#00h
	call	put_lo38k	;XX	 max until 128byte data @ 80hex byte
	ret
;======================Endurance========================================
endurance_bist_38k:
	mov 	r2,#5				;5 character to be sent
	mov		A,#00h				;1mc
	call	put_lo38k			;2mc
	mov		A,#60h
	call	put_lo38k
xx:
	mov		A, 63h				 ;mov data stored in 63h to acc
	jbc		23h,put_hi_xx		 ;if bit=1, goto put_hi_xx
	call	put_lo38k			 ;otherwise
	jmp		yy					 
put_hi_xx:
	call	put_hi38k			
yy:
	mov		a, 62h				  ;mov data stored in 62h to acc
	jbc		24h,put_hi_yy		  ;if bit=1, goto put_hi_yy
	call	put_lo38k			  ;otherwise
	jmp		next_byte
put_hi_yy:
	call	put_hi38k
next_byte:	
	mov		a,#00h
	call	put_lo38k
	ret

;======================Math-Co========================================	
MathCo_512bit_38k:
	mov 	r2,#5	;5 character to be sent
	mov		A,#00h	;1mc
	call	put_lo38k	;2mc
	mov		A,#4Ah
	call	put_hi38k
	mov		A,#00h
	call	put_lo38k	;msb
	mov		a,#00h
	call	put_lo38k	;lsb
	mov		a,#00h
	call	put_lo38k	;XX	 max until 128byte data @ 80hex byte
	ret
;======================Switch Clock========================================
switch_clock_38k:
	mov 	r2,#5				;5 character to be sent
	mov		A,#00h				;1mc
	call	put_lo38k			;2mc
	mov		A,#4Fh
	call	put_hi38k
switch_input:
	mov		A,63h							 
	jbc		23h,put_hi_parity
	call	put_lo38k
	jmp		nex
put_hi_parity:
	call	put_hi38k
nex:										 
	mov		a,62h
	jbc		24h,put_hi_parity2
	call	put_lo38k
	jmp		nex2
put_hi_parity2:
	call	put_hi38k
next2:
	mov		a,#00h
	call	put_lo38k
	ret


;20.2752MHz, M=16h, N=10h	  ;work
;23.9616MHz, M=1Ah, N=10h
;24.8832MHz, M=1Bh, N=10h
;22.1184MHz, M=18h, N=10h
;23.04		, M=19h, N=10h
;===========================OTP=======================================

OTP_CRC_additional38k:								;OTP CRC using MOVC
	mov 	r2,#5	
	mov		A,#00h	
	call	put_lo38k	
	mov		A,#35h
	call	put_lo38k
	mov		A,#00h
	call	put_lo38k
	mov		a,#00h
	call	put_lo38k
	mov		a,#02h
	call	put_hi38k
	ret
													
OTP_CRC_38k:
	mov r2,#5	;5 character to be sent
	mov	A,#00h	;1mc
	call	put_lo38k	;2mc
	mov	A,#34h
	call	put_hi38k
	mov	A,#00h
	call	put_lo38k
	mov	a,#00h
	call	put_lo38k
	mov	a,#02h
	call	put_hi38k
	ret
;====================================RSA===================================
RSA_512b_38k:								;RSA 512 bit
	mov 	r2,#5	
	mov		A,#00h	
	lcall	put_lo38k	
	mov		A,#40h
	lcall	put_hi38k
	mov		A,#00h
	lcall	put_lo38k
	mov		a,#00h
	lcall	put_lo38k
	mov		a,#00h
	lcall	put_hi38k
	ret
;=====================================================================
;===============================115k mode=============================
;=====================================================================
/*
SramBist115k:
	mov r2,#5	;5 character to be sent
	mov	A,#00h	;1mc
	call	put_lo115k	;2mc
	mov	A,#0Ah
	call	put_lo115k
	mov	A,#00h
	call	put_lo115k
	mov	a,#00h
	call	put_lo115k
	mov	a,#00h
	call	put_lo115k
	ret

erase115k:
	mov r2,#5	;5 character to be sent
	mov	A,#00h	;1mc
	lcall	put_lo115k	;2mc
	mov	A,#10h
	lcall	put_hi115k
	mov	A,#00h
	lcall	put_lo115k
	mov	a,#00h
	lcall	put_lo115k
	mov	a,#00h
	lcall	put_lo115k	
	ret

write115k:
	mov r2,#5	;5 character to be sent
	mov	a,#00h	;1mc
	lcall	put_lo115k	;2mc
	mov	a,#12h
	lcall	put_lo115k
	mov	a,#00h
	lcall	put_lo115k
	mov	a,#00h
	lcall	put_lo115k
	mov	a,#00h
	lcall	put_lo115k
	ret

;======================Math-Co========================================	
MathCo_512bit_115k:
	mov 	r2,#5	;5 character to be sent
	mov		A,#00h	;1mc
	call	put_lo115k	;2mc
	mov		A,#4Ah
	call	put_hi115k
	mov		A,#00h
	call	put_lo115k	;msb
	mov		a,#00h
	call	put_lo115k	;lsb
	mov		a,#00h
	call	put_lo115k	;XX	 max until 128byte data @ 80hex byte
	ret

;======================Switch Clock========================================
switch_clock_115k:
	mov 	r2,#5				;5 character to be sent
	mov		A,#00h				;1mc
	call	put_lo115k			;2mc
	mov		A,#4Fh
	call	put_hi115k
switch_input:
	mov		A,63h							 ;NeoX3 PLL Table
	jbc		23h,put_hi_parity
	call	put_lo115k
	jmp		nex
put_hi_parity:
	call	put_hi115k
nex:										 ;20.2752MHz, M=16h, N=10h	  ;work
	mov		a,#10h							 ;23.9616MHz, M=1Ah, N=10h
	call	put_hi115k						 ;24.8832MHz, M=1Bh, N=10h
	mov		a,#00h							 ;22.1184MHz, M=18h, N=10h
	call	put_lo115k						 ;23.04		, M=19h, N=10h
	ret

;============================================================================
writeeven115k:
	mov r2,#6	;5 character to be sent
	mov	a,#00h	;1mc
	call	put_lo115k	;2mc
	mov	a,#14h
	call	put_lo115k
	mov	a,#00h
	call	put_lo115k
	mov	a,#00h
	call	put_lo115k
	mov	a,#01h
	call	put_hi115k
data_byte:
	mov		A,63h							 
	jbc		23h,data_byte_hi
	call	put_lo115k
	jmp		nex2
data_byte_hi:
	call	put_hi115k
nex2:					
	;mov	a,#0AAh
	;lcall	put_lo115k
	ret

writeodd115k:
	mov r2,#6	;5 character to be sent
	mov	a,#00h	;1mc
	call	put_lo115k	;2mc
	mov	a,#16h
	call	put_hi115k
	mov	a,#00h
	call	put_lo115k
	mov	a,#00h
	call	put_lo115k
	mov	a,#01h
	call	put_hi115k
data_byte2:
	mov		A,63h							 
	jbc		23h,data_byte_hi2
	call	put_lo115k
	jmp		nex3
data_byte_hi2:
	call	put_hi115k
	;mov	a,#55h
	;lcall	put_lo115k
nex3:
	ret

CPU115k:
	mov r2,#5	;5 character to be sent
	mov	A,#00h	;1mc
	lcall	put_lo115k	;2mc
	mov	A,#02h
	lcall	put_hi115k
	mov	A,#00h
	lcall	put_lo115k
	mov	a,#00h
	lcall	put_lo115k
	mov	a,#00h
	lcall	put_lo115k
	ret

checksum115k:
	mov r2,#5	;5 character to be sent
	mov	a,#00h	;1mc
	lcall	put_lo115k	;2mc
	mov	a,#1Eh
	lcall	put_lo115k
	mov	a,#00h
	lcall	put_lo115k
	mov	a,#00h
	lcall	put_lo115k
	mov	a,#02h
	lcall	put_hi115k
	ret

keygen115k:
	mov r2,#5	;5 character to be sent
	mov	A,#00h	;1mc
	lcall	put_lo115k	;2mc
	mov	A,#01h
	lcall	put_hi115k
	mov	A,#00h
	lcall	put_lo115k	;msb
	mov	a,#00h
	lcall	put_lo115k	;lsb
	mov	a,#00h
	lcall	put_lo115k	;XX	 max until 128byte data @ 80hex byte
	ret

;===========================OTP=======================================

OTP_CRC_additional115k:								;OTP CRC using MOVC
	mov 	r2,#5	
	mov		A,#00h	
	call	put_lo115k	
	mov		A,#35h
	call	put_lo115k
	mov		A,#00h
	call	put_lo115k
	mov		a,#00h
	call	put_lo115k
	mov		a,#02h
	call	put_hi115k
	ret
													
OTP_CRC_115k:
	mov r2,#5	;5 character to be sent
	mov	A,#00h	;1mc
	call	put_lo115k	;2mc
	mov	A,#34h
	call	put_hi115k
	mov	A,#00h
	call	put_lo115k
	mov	a,#00h
	call	put_lo115k
	mov	a,#02h
	call	put_hi115k
	ret
;====================================RSA===================================
RSA_512b_115k:								;RSA 512 bit
	mov 	r2,#5	
	mov		A,#00h	
	lcall	put_lo115k	
	mov		A,#40h
	lcall	put_hi115k
	mov		A,#00h
	lcall	put_lo115k
	mov		a,#00h
	lcall	put_lo115k
	mov		a,#00h
	lcall	put_hi115k
	ret

;======================Endurance========================================
endurance_bist_115k:
	mov 	r2,#5				;5 character to be sent
	mov		A,#00h				;1mc
	call	put_lo115k			;2mc
	mov		A,#60h
	call	put_lo115k
xx:
	mov		A, 63h
	jbc		23h,put_hi_xx
	call	put_lo115k
	jmp		yy
put_hi_xx:
	call	put_hi115k			
yy:
	mov		a, 62h
	jbc		24h,put_hi_yy
	call	put_lo115k			
	jmp		next_byte
put_hi_yy:
	call	put_hi115k
next_byte:	
	mov		a,#00h
	call	put_lo115k
	ret

Timer115k:
	mov 	r2,#5	;5 character to be sent
	mov		A,#00h	;1mc
	call	put_lo115k	;2mc
	mov		A,#04h
	call	put_hi115k
	mov		A,#00h
	call	put_lo115k
	mov		a,#00h
	call	put_lo115k
	mov		a,#00h
	call	put_lo115k
	ret

Watchdog115k:
	mov 	r2,#5	;5 character to be sent
	mov		A,#00h	;1mc
	call	put_lo115k	;2mc
	mov		A,#06h
	call	put_lo115k
	mov		A,#00h
	call	put_lo115k
	mov		a,#00h
	call	put_lo115k
	mov		a,#00h
	call	put_lo115k
	ret

DES_CBC115k:
	mov 	r2,#5	;5 character to be sent
	mov		A,#00h	;1mc
	call	put_lo115k	;2mc
	mov		A,#44h
	call	put_lo115k
	mov		A,#00h
	call	put_lo115k
	mov		a,#00h
	call	put_lo115k
	mov		a,#00h
	call	put_lo115k
	ret

DES_nonCBC115k:
	mov 	r2,#5	;5 character to be sent
	mov		A,#00h	;1mc
	call	put_lo115k	;2mc
	mov		A,#46h
	call	put_hi115k
	mov		A,#00h
	call	put_lo115k
	mov		a,#00h
	call	put_lo115k
	mov		a,#00h
	call	put_lo115k
	ret
*/
;================include file==========
	$include (transmit_program.asm)
	$include (status_program.asm)	
	$include (count_program.asm)
	$include (lcd_program.asm)
	$include (ATR_program.asm)
;======================================
end